STORE_NAME = '.kaboom.json'
HELP = '''
Usage ---  kaboom [-h] action [...]
  kaboom: help -------------------------------------------------------

  kaboom                        this help text
  kaboom help                   this help text

  kaboom add <name> <value>     Add name:value to kaboom

  kaboom show <name>            display value of name
  kaboom show all               display everything stored in kaboom

  kaboom delete <name>          Remove key <name>
  kaboom delete all             Remove all keys

  kaboom update <name> <value>  Updates name's value to supplied value

  all other documentation is located at:
    https://github.com/vinu76jsr/kaboom
  Inspired by https://github.com/holman/boom'''